package org.protelis.parser.validation;

import com.google.common.base.Objects;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.protelis.parser.protelis.Block;
import org.protelis.parser.protelis.FunctionDef;
import org.protelis.parser.protelis.Lambda;
import org.protelis.parser.protelis.ProtelisPackage;
import org.protelis.parser.protelis.Statement;
import org.protelis.parser.protelis.VarDef;
import org.protelis.parser.protelis.VarDefList;
import org.protelis.parser.validation.AbstractProtelisValidator;

/**
 * Custom validation rules.
 * 
 * see http://www.eclipse.org/Xtext/documentation.html#validation
 */
@SuppressWarnings("all")
public class ProtelisValidator extends AbstractProtelisValidator {
  /**
   * Make sure that nobody defined the variable already:
   * 
   * other previous lets;
   * 
   * containing function;
   * 
   * containing lambda;
   * 
   * containing rep
   */
  @Check
  public void letNameDoesNotShadowArguments(final VarDef exp) {
    EObject parent = exp.eContainer();
    while ((parent != null)) {
      {
        if ((parent instanceof Block)) {
          if (((((Block)parent).getFirst() instanceof VarDef) && (!Objects.equal(((Block)parent).getFirst(), exp)))) {
            Statement _first = ((Block)parent).getFirst();
            final VarDef otherLet = ((VarDef) _first);
            boolean _equals = otherLet.getName().equals(exp.getName());
            if (_equals) {
              InputOutput.<String>print("CAIONE");
              this.error(exp);
            }
          }
        }
        if ((parent instanceof FunctionDef)) {
          VarDefList _args = ((FunctionDef)parent).getArgs();
          boolean _tripleNotEquals = (_args != null);
          if (_tripleNotEquals) {
            final Function1<VarDef, String> _function = (VarDef it) -> {
              return it.getName();
            };
            boolean _contains = ListExtensions.<VarDef, String>map(((FunctionDef)parent).getArgs().getArgs(), _function).contains(exp.getName());
            if (_contains) {
              this.error(exp);
            }
          }
        }
        if ((parent instanceof Lambda)) {
          EObject _args_1 = ((Lambda)parent).getArgs();
          boolean _tripleNotEquals_1 = (_args_1 != null);
          if (_tripleNotEquals_1) {
            final EObject args = ((Lambda)parent).getArgs();
            if ((args instanceof VarDef)) {
              boolean _equals_1 = ((VarDef)args).getName().equals(exp.getName());
              if (_equals_1) {
                this.error(exp);
              }
            } else {
              if ((args instanceof VarDefList)) {
                final Function1<VarDef, String> _function_1 = (VarDef it) -> {
                  return it.getName();
                };
                boolean _contains_1 = ListExtensions.<VarDef, String>map(((VarDefList)args).getArgs(), _function_1).contains(exp.getName());
                if (_contains_1) {
                  this.error(exp);
                }
              }
            }
          }
        }
        parent = parent.eContainer();
      }
    }
  }
  
  public void error(final VarDef exp) {
    final String error = "The variable has already been defined in this context. Pick another name.";
    EObject _eContainer = exp.eContainer();
    if ((_eContainer instanceof Block)) {
      this.error(error, exp.eContainer(), ProtelisPackage.Literals.BLOCK__FIRST);
    }
  }
}
